/*************************************************
*                                                *
*  EasyBMP Cross-Platform Windows Bitmap Library * 
*                                                *
*  Author: Paul Macklin                          *
*   email: pmacklin@math.uci.edu                 *
*                                                *
*    file: EasyBMP_StandardColorTables.h         *
*    date: 5-2-2005                              *
* version: 0.65                                  *
*                                                *
* description: Assists in writing color tables   *
*              for the BMP class                 *
*                                                *
*************************************************/

#ifndef _EasyBMP_StandardColorTables_h_
#define _EasyBMP_StandardColorTables_h_

void CreateGreyscaleColorTable( RGBApixel** pColorPointer , int Depth )
{
 if( Depth != 1 && Depth != 4 && Depth != 8 )
 { return; }
 int i;
 int NumberOfColors = IntPow( 2, Depth);
 BYTE StepSize;
 if( Depth != 1 )
 { StepSize = 255/(NumberOfColors-1); }
 else
 { StepSize = 255; }
 
 for( i=0 ; i < NumberOfColors ; i++ )
 {
  BYTE TempColor = i*StepSize;
  (*pColorPointer)[i].Red = TempColor;
  (*pColorPointer)[i].Green = TempColor;
  (*pColorPointer)[i].Blue = TempColor;
  (*pColorPointer)[i].Alpha = 0;
 }
 
 return;
}

void Create1bitColorTable( RGBApixel** pColorPointer )
{
 int i;
 for( i=0 ; i < 2 ; i++ )
 {
  (*pColorPointer)[i].Red = i*255;
  (*pColorPointer)[i].Green = i*255;
  (*pColorPointer)[i].Blue = i*255;
  (*pColorPointer)[i].Alpha = 0;
 } 
} 

void Create4bitColorTable( RGBApixel** pColorPointer )
{
 int i = 0;
 int j,k,ell;
 
 // simplify the code for the first 8 colors
 for( ell=0 ; ell < 2 ; ell++ )
 {
  for( k=0 ; k < 2 ; k++ )
  {
   for( j=0 ; j < 2 ; j++ )
   {
    (*pColorPointer)[i].Red = j*128;
	(*pColorPointer)[i].Green = k*128;
	(*pColorPointer)[i].Blue = ell*128;
	i++;
   }
  }
 }
 
 // simplify the code for the last 8 colors
 for( ell=0 ; ell < 2 ; ell++ )
 {
  for( k=0 ; k < 2 ; k++ )
  {
   for( j=0 ; j < 2 ; j++ )
   {
    (*pColorPointer)[i].Red = j*255;
	(*pColorPointer)[i].Green = k*255;
	(*pColorPointer)[i].Blue = ell*255;
	i++;
   }
  }
 }
 
 // overwrite the duplicate color
 i=8; 
 (*pColorPointer)[i].Red = 192;
 (*pColorPointer)[i].Green = 192;
 (*pColorPointer)[i].Blue = 192;
 
 for( i=0 ; i < 16 ; i++ )
 {
  (*pColorPointer)[i].Alpha = 0;
 }
  
 return;
}

void Create8bitColorTable( RGBApixel** pColorPointer )
{
 int i=0;
 int j,k,ell;
 
 // do an easy loop, which works for all but colors 
 // 0 to 9 and 246 to 255
 for( ell=0 ; ell < 4 ; ell++ ) 
 {
  for( k=0 ; k < 8 ; k++ )
  {
   for( j=0; j < 8 ; j++ )
   {
    (*pColorPointer)[i].Red = j*32;
	(*pColorPointer)[i].Green = k*32;
	(*pColorPointer)[i].Blue = ell*64;
	(*pColorPointer)[i].Alpha = 0;
	i++;
   }
  }
 } 
 
 // now redo the first 8 colors  
 i=0;
 for( ell=0 ; ell < 2 ; ell++ ) 
 {
  for( k=0 ; k < 2 ; k++ )
  {
   for( j=0; j < 2 ; j++ )
   {
    (*pColorPointer)[i].Red = j*128;
	(*pColorPointer)[i].Green = k*128;
	(*pColorPointer)[i].Blue = ell*128;
	i++;
   }
  }
 } 
 
 // overwrite colors 7, 8, 9
 i=7;
 (*pColorPointer)[i].Red = 192;
 (*pColorPointer)[i].Green = 192;
 (*pColorPointer)[i].Blue = 192;
 i++; // 8
 (*pColorPointer)[i].Red = 192;
 (*pColorPointer)[i].Green = 220;
 (*pColorPointer)[i].Blue = 192;
 i++; // 9
 (*pColorPointer)[i].Red = 166;
 (*pColorPointer)[i].Green = 202;
 (*pColorPointer)[i].Blue = 240;

 // overwrite colors 246 to 255 
 i=246;
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 251;
 (*pColorPointer)[i].Blue = 240;
 i++; // 247
 (*pColorPointer)[i].Red = 160;
 (*pColorPointer)[i].Green = 160;
 (*pColorPointer)[i].Blue = 164;
 i++; // 248
 (*pColorPointer)[i].Red = 128;
 (*pColorPointer)[i].Green = 128;
 (*pColorPointer)[i].Blue = 128;
 i++; // 249
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 0;
 i++; // 250
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 0;
 i++; // 251
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 0;
 i++; // 252
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 255;
 i++; // 253
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 255;
 i++; // 254
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 255;
 i++; // 255
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 255;

 return;
}

void CreateStandardColorTable(  RGBApixel** pColorPointer , int Depth )
{
 using namespace std;
 if( Depth == 1 )
 {
  Create1bitColorTable( pColorPointer );
  return;
 }
 if( Depth == 4 )
 {
  Create4bitColorTable( pColorPointer );
  return;
 }
 if( Depth == 8 )
 {
  Create8bitColorTable( pColorPointer );
  return;
 }
 cout << "EasyBMP Warning: Attempted to create color table with \n"
      << "                 unsupported bit depth " << Depth << ".\n"
	  << "                 Ignoring request.\n";
 return;
}

#endif
